#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
colors.py is a part of colored.

Copyright 2014-2017 Dimitris Zlatanidis <d.zlatanidis@gmail.com>
All rights reserved.

Colored is very simple Python library for color and formatting in terminal.

https://github.com/dslackw/colored

colored is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


names = [
    'BLACK',
    'RED',
    'GREEN',
    'YELLOW',
    'BLUE',
    'MAGENTA',
    'CYAN',
    'LIGHT_GRAY',
    'DARK_GRAY',
    'LIGHT_RED',
    'LIGHT_GREEN',
    'LIGHT_YELLOW',
    'LIGHT_BLUE',
    'LIGHT_MAGENTA',
    'LIGHT_CYAN',
    'WHITE',
    'GREY_0',
    'NAVY_BLUE',
    'DARK_BLUE',
    'BLUE_3A',
    'BLUE_3B',
    'BLUE_1',
    'DARK_GREEN',
    'DEEP_SKY_BLUE_4A',
    'DEEP_SKY_BLUE_4B',
    'DEEP_SKY_BLUE_4C',
    'DODGER_BLUE_3',
    'DODGER_BLUE_2',
    'GREEN_4',
    'SPRING_GREEN_4',
    'TURQUOISE_4',
    'DEEP_SKY_BLUE_3A',
    'DEEP_SKY_BLUE_3B',
    'DODGER_BLUE_1',
    'GREEN_3A',
    'SPRING_GREEN_3A',
    'DARK_CYAN',
    'LIGHT_SEA_GREEN',
    'DEEP_SKY_BLUE_2',
    'DEEP_SKY_BLUE_1',
    'GREEN_3B',
    'SPRING_GREEN_3B',
    'SPRING_GREEN_2A',
    'CYAN_3',
    'DARK_TURQUOISE',
    'TURQUOISE_2',
    'GREEN_1',
    'SPRING_GREEN_2B',
    'SPRING_GREEN_1',
    'MEDIUM_SPRING_GREEN',
    'CYAN_2',
    'CYAN_1',
    'DARK_RED_1',
    'DEEP_PINK_4A',
    'PURPLE_4A',
    'PURPLE_4B',
    'PURPLE_3',
    'BLUE_VIOLET',
    'ORANGE_4A',
    'GREY_37',
    'MEDIUM_PURPLE_4',
    'SLATE_BLUE_3A',
    'SLATE_BLUE_3B',
    'ROYAL_BLUE_1',
    'CHARTREUSE_4',
    'DARK_SEA_GREEN_4A',
    'PALE_TURQUOISE_4',
    'STEEL_BLUE',
    'STEEL_BLUE_3',
    'CORNFLOWER_BLUE',
    'CHARTREUSE_3A',
    'DARK_SEA_GREEN_4B',
    'CADET_BLUE_2',
    'CADET_BLUE_1',
    'SKY_BLUE_3',
    'STEEL_BLUE_1A',
    'CHARTREUSE_3B',
    'PALE_GREEN_3A',
    'SEA_GREEN_3',
    'AQUAMARINE_3',
    'MEDIUM_TURQUOISE',
    'STEEL_BLUE_1B',
    'CHARTREUSE_2A',
    'SEA_GREEN_2',
    'SEA_GREEN_1A',
    'SEA_GREEN_1B',
    'AQUAMARINE_1A',
    'DARK_SLATE_GRAY_2',
    'DARK_RED_2',
    'DEEP_PINK_4B',
    'DARK_MAGENTA_1',
    'DARK_MAGENTA_2',
    'DARK_VIOLET_1A',
    'PURPLE_1A',
    'ORANGE_4B',
    'LIGHT_PINK_4',
    'PLUM_4',
    'MEDIUM_PURPLE_3A',
    'MEDIUM_PURPLE_3B',
    'SLATE_BLUE_1',
    'YELLOW_4A',
    'WHEAT_4',
    'GREY_53',
    'LIGHT_SLATE_GREY',
    'MEDIUM_PURPLE',
    'LIGHT_SLATE_BLUE',
    'YELLOW_4B',
    'DARK_OLIVE_GREEN_3A',
    'DARK_GREEN_SEA',
    'LIGHT_SKY_BLUE_3A',
    'LIGHT_SKY_BLUE_3B',
    'SKY_BLUE_2',
    'CHARTREUSE_2B',
    'DARK_OLIVE_GREEN_3B',
    'PALE_GREEN_3B',
    'DARK_SEA_GREEN_3A',
    'DARK_SLATE_GRAY_3',
    'SKY_BLUE_1',
    'CHARTREUSE_1',
    'LIGHT_GREEN_2',
    'LIGHT_GREEN_3',
    'PALE_GREEN_1A',
    'AQUAMARINE_1B',
    'DARK_SLATE_GRAY_1',
    'RED_3A',
    'DEEP_PINK_4C',
    'MEDIUM_VIOLET_RED',
    'MAGENTA_3A',
    'DARK_VIOLET_1B',
    'PURPLE_1B',
    'DARK_ORANGE_3A',
    'INDIAN_RED_1A',
    'HOT_PINK_3A',
    'MEDIUM_ORCHID_3',
    'MEDIUM_ORCHID',
    'MEDIUM_PURPLE_2A',
    'DARK_GOLDENROD',
    'LIGHT_SALMON_3A',
    'ROSY_BROWN',
    'GREY_63',
    'MEDIUM_PURPLE_2B',
    'MEDIUM_PURPLE_1',
    'GOLD_3A',
    'DARK_KHAKI',
    'NAVAJO_WHITE_3',
    'GREY_69',
    'LIGHT_STEEL_BLUE_3',
    'LIGHT_STEEL_BLUE',
    'YELLOW_3A',
    'DARK_OLIVE_GREEN_3',
    'DARK_SEA_GREEN_3B',
    'DARK_SEA_GREEN_2',
    'LIGHT_CYAN_3',
    'LIGHT_SKY_BLUE_1',
    'GREEN_YELLOW',
    'DARK_OLIVE_GREEN_2',
    'PALE_GREEN_1B',
    'DARK_SEA_GREEN_5B',
    'DARK_SEA_GREEN_5A',
    'PALE_TURQUOISE_1',
    'RED_3B',
    'DEEP_PINK_3A',
    'DEEP_PINK_3B',
    'MAGENTA_3B',
    'MAGENTA_3C',
    'MAGENTA_2A',
    'DARK_ORANGE_3B',
    'INDIAN_RED_1B',
    'HOT_PINK_3B',
    'HOT_PINK_2',
    'ORCHID',
    'MEDIUM_ORCHID_1A',
    'ORANGE_3',
    'LIGHT_SALMON_3B',
    'LIGHT_PINK_3',
    'PINK_3',
    'PLUM_3',
    'VIOLET',
    'GOLD_3B',
    'LIGHT_GOLDENROD_3',
    'TAN',
    'MISTY_ROSE_3',
    'THISTLE_3',
    'PLUM_2',
    'YELLOW_3B',
    'KHAKI_3',
    'LIGHT_GOLDENROD_2A',
    'LIGHT_YELLOW_3',
    'GREY_84',
    'LIGHT_STEEL_BLUE_1',
    'YELLOW_2',
    'DARK_OLIVE_GREEN_1A',
    'DARK_OLIVE_GREEN_1B',
    'DARK_SEA_GREEN_1',
    'HONEYDEW_2',
    'LIGHT_CYAN_1',
    'RED_1',
    'DEEP_PINK_2',
    'DEEP_PINK_1A',
    'DEEP_PINK_1B',
    'MAGENTA_2B',
    'MAGENTA_1',
    'ORANGE_RED_1',
    'INDIAN_RED_1C',
    'INDIAN_RED_1D',
    'HOT_PINK_1A',
    'HOT_PINK_1B',
    'MEDIUM_ORCHID_1B',
    'DARK_ORANGE',
    'SALMON_1',
    'LIGHT_CORAL',
    'PALE_VIOLET_RED_1',
    'ORCHID_2',
    'ORCHID_1',
    'ORANGE_1',
    'SANDY_BROWN',
    'LIGHT_SALMON_1',
    'LIGHT_PINK_1',
    'PINK_1',
    'PLUM_1',
    'GOLD_1',
    'LIGHT_GOLDENROD_2B',
    'LIGHT_GOLDENROD_2C',
    'NAVAJO_WHITE_1',
    'MISTY_ROSE1',
    'THISTLE_1',
    'YELLOW_1',
    'LIGHT_GOLDENROD_1',
    'KHAKI_1',
    'WHEAT_1',
    'CORNSILK_1',
    'GREY_100',
    'GREY_3',
    'GREY_7',
    'GREY_11',
    'GREY_15',
    'GREY_19',
    'GREY_23',
    'GREY_27',
    'GREY_30',
    'GREY_35',
    'GREY_39',
    'GREY_42',
    'GREY_46',
    'GREY_50',
    'GREY_54',
    'GREY_58',
    'GREY_62',
    'GREY_66',
    'GREY_70',
    'GREY_74',
    'GREY_78',
    'GREY_82',
    'GREY_85',
    'GREY_89',
    'GREY_93'
    ]
